#ifndef __JTVREMSK_H
#define __JTVREMSK_H

struct KeyDef
{
	DWORD code;
	char name[32];
	char alt_name[32];
};

class CRemoteDriver : public IJTVRemoteControl
{
protected:
	HANDLE hThread;
	DWORD dwRefCount;
	HANDLE hBreakEvent;
	DWORD last_tick;
	
	IUnknown *device;
	IJTVRemoteHost *host;
	KeyDef *keys;
	int num_keys;

public:
	CRemoteDriver(KeyDef *akeys, int count)
	{
		dwRefCount = 1;
		keys = akeys;
		num_keys = count;
		device = 0;
		host = 0;
	};

	virtual ~CRemoteDriver()
	{
		Unadvise();
	};

	HRESULT STDMETHODCALLTYPE QueryInterface(REFIID iid, LPVOID *result)
	{
		if (iid == IID_IUnknown || iid == IID_IJTVRemoteControl)
		{
			*result = (IJTVRemoteControl *)this;
			AddRef();
			return S_OK;
		};
		return E_NOINTERFACE;
	};

	ULONG STDMETHODCALLTYPE AddRef()
	{
		return ++dwRefCount;
	};

	ULONG STDMETHODCALLTYPE Release()
	{
		if (!--dwRefCount)
		{
			delete this;
			return 0;
		}
		else
			return dwRefCount;
	};

	STDMETHODIMP Advise(IJTVRemoteHost *ahost, IUnknown *dev)
	{
		device = dev;
		if (device)
		{
			DWORD id;

			device->AddRef();
			host = ahost;
			if (host)
			{
				host->AddRef();
			};

			hBreakEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
			hThread = CreateThread(NULL, 32768, PollingThread, this, 0, &id);

			return (hThread != 0) ? S_OK : E_UNEXPECTED;
		}
		return E_UNEXPECTED;
	};

	STDMETHODIMP Unadvise()
	{
		if (device)
		{
			SetEvent(hBreakEvent);
			WaitForSingleObject(hThread, 5000);
			CloseHandle(hThread);
			CloseHandle(hBreakEvent);

			device->Release();
			device = 0;
		};
		if (host)
		{
			host->Release();
			host = 0;
		};
		return S_OK;
	};
	virtual void STDMETHODCALLTYPE Reinit() 
	{
	};

	void AddStroke(DWORD val)
	{
		if (!host)
			return;

		for (int i = 0; i < num_keys; ++i)
		{
			if (val == keys[i].code)
			{
				host->KeyCallback(keys[i].name, keys[i].alt_name);
				return;
			};
		};
	};

private:

	virtual void PollingThread() = 0;

	static DWORD WINAPI PollingThread(LPVOID param)
	{
		((CRemoteDriver *)param)->PollingThread();
		return 0;
	};

};

#endif
